package meta

import (
	"github.com/ankitrgadiya/operatorlib/pkg/interfaces"
	"github.com/pkg/errors"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
)

// GenerateObjectMeta function generates ObjectMeta struct as per the
// Conf struct passed.
func MutateObjectMeta(g Mutate) (om *metav1.ObjectMeta, err error) {
	var labels, annotations map[string]string
	var finalizers []string

	labels, err = g.GenLabelsFunc()
	if err != nil {
		return nil, errors.Wrap(err, "failed to generate labels")
	}

	//	if g.AppendLabels {
	//		err = mergo.Merge(&labels, g.Instance.GetLabels())
	//		if err != nil {
	//			return nil, errors.Wrap(err, "failed to append labels from owner object")
	//		}
	//	}

	annotations, err = g.GenAnnotationsFunc()
	if err != nil {
		return nil, errors.Wrap(err, "failed to generate annotations")
	}

	finalizers, err = g.GenFinalizersFunc()
	if err != nil {
		return nil, errors.Wrap(err, "failed to generate finalizers")
	}

	om = &metav1.ObjectMeta{
		Name:        g.GetName(),
		Namespace:   g.GetNamespace(),
		Labels:      labels,
		Annotations: annotations,
		Finalizers:  finalizers,
	}

	return om, nil
}

// NewObjectSyncer creates a new kubernetes.Object syncer for a given object
// with an owner and persists data using controller-runtime's CreateOrUpdate.
// The name is used for logging and event emitting purposes and should be an
// valid go identifier in upper camel case. (eg. MysqlStatefulSet)
func NewMetaMutate(name, namespace string, owner interfaces.Object, ownerRef bool, genLabels GenLabels) Mutate {
	return &Conf{
		Instance:       owner,
		Namespace:      namespace,
		Name:           name,
		OwnerReference: ownerRef,
		GenLabels:      genLabels,
	}
}

func (c Conf) GetInstance() interfaces.Object {
	return c.Instance
}
func (c Conf) GetName() string {
	return c.Name
}
func (c Conf) GetNamespace() string {
	return c.Namespace
}

func (c Conf) GenLabelsFunc() (map[string]string, error) {
	var labels map[string]string
	if c.GenLabels != nil {
		return c.GenLabels(c.Instance)
	}
	return labels, nil
}
func (c Conf) GenAnnotationsFunc() (map[string]string, error) {
	var annotations map[string]string
	if c.GenAnnotations != nil {
		return c.GenAnnotations(c.Instance)
	}
	return annotations, nil

}
func (c Conf) GenFinalizersFunc() ([]string, error) {
	if c.GenFinalizers != nil {
		return c.GenFinalizers(c.Instance)
	}
	return nil, nil

}
